@@ -4,11 +4,11 @@ require Tk;
 
 
 use vars qw($VERSION);
-$VERSION = '2.004'; # $Id: //depot/Tk/Extensions/ImageBack/ImageBack.pm#4$
+$VERSION = '3.004'; # $Id: //depot/Tk8/Extensions/ImageBack/ImageBack.pm#4$
 
 @ISA = qw(DynaLoader);
 
-bootstrap Tk::ImageBack;
+bootstrap Tk::ImageBack $Tk::VERSION;
 
 1;
 __END__
