@@ -1,16 +1,16 @@
-package Tk::HList;
+package Tk::HList; 
 
-use vars qw($VERSION);
-$VERSION = '3.030'; # $Id: //depot/Tk8/HList/HList.pm#30 $
+use vars qw($VERSION @ISA);
+$VERSION = '3.011'; # $Id: //depot/Tk8/HList/HList.pm#11$
 
-use Tk qw(Ev $XS_VERSION);
+use Tk qw(Ev);
 
-use base  qw(Tk::Widget);
+@ISA = qw(Tk::Widget);
 
 Construct Tk::Widget 'HList';
 sub Tk::Widget::ScrlHList { shift->Scrolled('HList'=>@_) }
 
-bootstrap Tk::HList;
+bootstrap Tk::HList $Tk::VERSION; 
 
 sub Tk_cmd { \&Tk::hlist }
 
@@ -18,27 +18,25 @@ sub CreateArgs
 {
  my ($package,$parent,$args) = @_;
  my @result = $package->SUPER::CreateArgs($parent,$args);
- my $columns = delete $args->{-columns};
+ my $columns = delete $args->{-columns};                     
  push(@result, '-columns' => $columns) if (defined $columns);
  return @result;
 }
 
-Tk::Methods qw(add addchild anchor column
-               delete dragsite dropsite entrycget
-               entryconfigure geometryinfo indicator header hide item info
-               nearest see select selection show xview yview);
+EnterMethods Tk::HList __FILE__,qw(add addchild anchor column
+                                   delete dragsite dropsite entrycget
+                                   entryconfigure geometryinfo indicator header hide item info
+                                   nearest see select selection show xview yview);
 
 use Tk::Submethods ( 'delete' => [qw(all entry offsprings siblings)],
                      'header' => [qw(configure cget create delete exists size)],
                      'indicator' => [qw(configure cget create delete exists size)],
                      'info' => [qw(anchor bbox children data dragsite
-                                   dropsite exists hidden item next parent prev
-                                   selection)],
+                                   dropsite exists hidden item next parent prev)],
                      'item' => [qw(configure cget create delete exists)],
                      'selection' => [qw(clear get includes set)],
                      'anchor' => [qw(clear set)],
                      'column' => [qw(width)],
-                     'hide' => [qw(entry)],
                    );
 
 
@@ -48,18 +46,34 @@ sub ClassInit
 
  $mw->bind($class,'<ButtonPress-1>',[ 'Button1' ] );
  $mw->bind($class,'<Shift-ButtonPress-1>',[ 'ShiftButton1' ] );
- $mw->bind($class,'<Control-ButtonRelease-1>','Control_ButtonRelease_1');
- $mw->bind($class,'<ButtonRelease-1>','ButtonRelease_1');
+ $mw->bind($class,'<Control-ButtonRelease-1>', sub {} );
+ $mw->bind($class,'<ButtonRelease-1>',
+		sub
+		 {
+		  my $w = shift;
+		  my $Ev = $w->XEvent;
+
+		  $w->CancelRepeat
+		      if($w->cget('-selectmode') ne "dragdrop");
+		  $w->ButtonRelease1($Ev);
+		 });
  $mw->bind($class,'<B1-Motion>',[ 'Button1Motion' ] );
  $mw->bind($class,'<B1-Leave>',[ 'AutoScan' ] );
 
  $mw->bind($class,'<Double-ButtonPress-1>',['Double1']);
 
- $mw->bind($class,'<Control-B1-Motion>','Control_B1_Motion');
+ $mw->bind($class,'<Control-B1-Motion>', sub {} );
  $mw->bind($class,'<Control-ButtonPress-1>',['CtrlButton1']);
  $mw->bind($class,'<Control-Double-ButtonPress-1>',['CtrlButton1']);
 
- $mw->bind($class,'<B1-Enter>','B1_Enter');
+ $mw->bind($class,'<B1-Enter>',
+		sub
+		 {
+		  my $w = shift;
+		  my $Ev = $w->XEvent;
+		  $w->CancelRepeat
+		      if($w->cget('-selectmode') ne "dragdrop");
+		 });
 
  $mw->bind($class,'<Up>',['UpDown', 'prev']);
  $mw->bind($class,'<Down>',['UpDown', 'next']);
@@ -69,80 +83,43 @@ sub ClassInit
 
  $mw->bind($class,'<Left>', ['LeftRight', 'left']);
  $mw->bind($class,'<Right>',['LeftRight', 'right']);
-
+                        
  $mw->PriorNextBind($class);
 
  $mw->bind($class,'<Return>', ['KeyboardActivate']);
  $mw->bind($class,'<space>',  ['KeyboardBrowse']);
  $mw->bind($class,'<Home>',   ['KeyboardHome']);
- $mw->bind($class,'<End>',    ['KeyboardEnd']);
 
  return $class;
 }
 
-sub Control_ButtonRelease_1
-{
-}
-
-
-sub ButtonRelease_1
-{
- my $w = shift;
- my $Ev = $w->XEvent;
- $w->CancelRepeat
- if($w->cget('-selectmode') ne 'dragdrop');
- $w->ButtonRelease1($Ev);
-}
-
-
-sub Control_B1_Motion
-{
-}
-
-
-sub B1_Enter
-{
- my $w = shift;
- my $Ev = $w->XEvent;
- $w->CancelRepeat
- if($w->cget('-selectmode') ne 'dragdrop');
-}
-
-
 sub Button1
 {
  my $w = shift;
  my $Ev = $w->XEvent;
 
- delete $w->{'shiftanchor'};
- delete $w->{tixindicator};
+ delete $w->{'shiftanchor'}; 
+ delete $w->{tixindicator}; 
 
  $w->focus()
-    if($w->cget('-takefocus'));
+    if($w->cget("-takefocus"));
 
- my $mode = $w->cget('-selectmode');
+ my $mode = $w->cget("-selectmode");
 
- if ($mode eq 'dragdrop')
+ if ($mode eq "dragdrop")
   {
    # $w->Send_WaitDrag($Ev->y);
    return;
   }
 
- my $ent = $w->GetNearest($Ev->y, 1);
-
- if (!defined($ent) || !length($ent))
-  {
-    $w->select('clear');
-    $w->anchor('clear');
-    return;
-  }
-
+ my $ent = $w->GetNearest($Ev->y);
 
+ return unless (defined($ent) and length($ent));
 
  my @info = $w->info('item',$Ev->x, $Ev->y);
  if (@info)
   {
-   die 'Assert' unless $info[0] eq $ent;
+   die "Assert" unless $info[0] eq $ent;
   }
  else
   {
@@ -152,38 +129,38 @@ sub Button1
  if (defined($info[1]) && $info[1] eq 'indicator')
   {
    $w->{tixindicator} = $ent;
-   $w->Callback(-indicatorcmd => $ent, '<Arm>');
+   $w->Callback(-indicatorcmd => $ent, "<Arm>");
   }
  else
   {
    my $browse = 0;
-
-   if($mode eq 'single')
+     
+   if($mode eq "single")
     {
      $w->anchor('set', $ent);
     }
-   elsif($mode eq 'browse')
+   elsif($mode eq "browse")
     {
      $w->anchor('set', $ent);
      $w->select('clear' );
      $w->select('set', $ent);
      $browse = 1;
     }
-   elsif($mode eq 'multiple')
+   elsif($mode eq "multiple")
     {
      $w->select('clear');
      $w->anchor('set', $ent);
      $w->select('set', $ent);
      $browse = 1;
     }
-   elsif($mode eq 'extended')
+   elsif($mode eq "extended")
     {
      $w->anchor('set', $ent);
      $w->select('clear');
      $w->select('set', $ent);
      $browse = 1;
     }
-
+     
    if ($browse)
     {
      $w->Callback(-browsecmd => @info);
@@ -196,19 +173,19 @@ sub ShiftButton1
  my $w = shift;
  my $Ev = $w->XEvent;
 
- my $to = $w->GetNearest($Ev->y, 1);
+ my $to = $w->GetNearest($Ev->y);
 
- delete $w->{'shiftanchor'};
- delete $w->{tixindicator};
+ delete $w->{'shiftanchor'}; 
+ delete $w->{tixindicator}; 
 
  return unless (defined($to) and length($to));
 
  my $mode = $w->cget('-selectmode');
 
- if($mode eq 'extended' or $mode eq 'multiple')
+ if($mode eq "extended")
   {
    my $from = $w->info('anchor');
-   if(defined $from)
+   if($from)
     {
      $w->select('clear');
      $w->select('set', $from, $to);
@@ -224,19 +201,10 @@ sub ShiftButton1
 
 sub GetNearest
 {
- my ($w,$y,$undefafterend) = @_;
+ my ($w,$y) = @_;
  my $ent = $w->nearest($y);
  if (defined $ent)
   {
-   if ($undefafterend)
-    {
-     my $borderwidth = $w->cget('-borderwidth');
-     my $highlightthickness = $w->cget('-highlightthickness');
-     my $bottomy = (split(' ', $w->infoBbox($ent)))[3];
-     $bottomy += $borderwidth + $highlightthickness;
-     if ($w->header('exist', 0)){ $bottomy += ($w->header('size', 0))[1]; };
-     if ($y > $bottomy){ return undef; }
-    }
    my $state = $w->entrycget($ent, '-state');
    return $ent if (!defined($state) || $state ne 'disabled');
   }
@@ -247,55 +215,57 @@ sub ButtonRelease1
 {
  my ($w, $Ev) = @_;
 
- delete $w->{'shiftanchor'};
+ delete $w->{'shiftanchor'}; 
 
  my $mode = $w->cget('-selectmode');
 
- if($mode eq 'dragdrop')
+ if($mode eq "dragdrop")
   {
 #   $w->Send_DoneDrag();
    return;
   }
 
  my ($x, $y) = ($Ev->x, $Ev->y);
- my $ent = $w->GetNearest($y, 1);
+ my $ent = $w->GetNearest($y);
 
- if (!defined($ent) and $mode eq 'single')
-  {
-     my $ent = $w->info('selection');
-     if (defined $ent)
-      {
-        $w->anchor('set', $ent);
-      }
-  }
  return unless (defined($ent) and length($ent));
 
- if(exists $w->{tixindicator})
+ if($w->{tixindicator})
   {
    return unless delete($w->{tixindicator}) eq $ent;
    my @info = $w->info('item',$Ev->x, $Ev->y);
    if(defined($info[1]) && $info[1] eq 'indicator')
     {
-     $w->Callback(-indicatorcmd => $ent, '<Activate>');
+     $w->Callback(-indicatorcmd => $ent, "<Activate>");
     }
    return;
   }
 
-  if($mode eq 'single' || $mode eq 'browse')
-   {
-    $w->anchor('set', $ent);
-    $w->select('clear');
-    $w->select('set', $ent);
+ if($x < 0 || $y < 0 || $x > $w->width || $y > $w->height)
+  {
+   $w->select('clear');
 
-   }
-  elsif($mode eq 'multiple')
-   {
-    $w->select('set', $ent);
-   }
-  elsif($mode eq 'extended')
-   {
-    $w->select('set', $ent);
-   }
+   return if($mode eq "single" || $mode eq "browse")
+
+  }
+ else
+  {
+   if($mode eq "single" || $mode eq "browse")
+    {
+     $w->anchor('set', $ent);
+     $w->select('clear');
+     $w->select('set', $ent);
+
+    }
+   elsif($mode eq "multiple")
+    {
+     $w->select('set', $ent);
+    }
+   elsif($mode eq "extended")
+    {
+     $w->select('set', $ent);
+    }
+  }
 
  $w->Callback(-browsecmd =>$ent);
 }
@@ -305,42 +275,34 @@ sub Button1Motion
  my $w = shift;
  my $Ev = $w->XEvent;
 
- delete $w->{'shiftanchor'};
+ delete $w->{'shiftanchor'}; 
 
  my $mode = $w->cget('-selectmode');
 
- if ($mode eq 'dragdrop')
+ if ($mode eq "dragdrop")
   {
 #   $w->Send_StartDrag();
    return;
   }
 
- my $ent;
- if (defined $w->info('anchor'))
-  {
-   $ent = $w->GetNearest($Ev->y);
-  }
- else
-  {
-   $ent = $w->GetNearest($Ev->y, 1);
-  }
+ my $ent = $w->GetNearest($Ev->y);
  return unless (defined($ent) and length($ent));
 
- if(exists $w->{tixindicator})
+ if($w->{tixindicator})
   {
-   my $event_type = $w->{tixindicator} eq $ent ? '<Arm>' : '<Disarm>';
+   my $event_type = $w->{tixindicator} eq $ent ? "<Arm>" : "<Disarm>";
    $w->Callback(-indicatorcmd => $w->{tixindicator}, $event_type );
    return;
   }
 
- if ($mode eq 'single')
+ if ($mode eq "single")
   {
    $w->anchor('set', $ent);
   }
- elsif ($mode eq 'multiple' || $mode eq 'extended')
+ elsif ($mode eq "multiple" || $mode eq "extended")
   {
    my $from = $w->info('anchor');
-   if(defined $from)
+   if($from)
     {
      $w->select('clear');
      $w->select('set', $from, $ent);
@@ -353,7 +315,7 @@ sub Button1Motion
     }
   }
 
- if ($mode ne 'single')
+ if ($mode ne "single")
   {
    $w->Callback(-browsecmd =>$ent);
   }
@@ -364,14 +326,14 @@ sub Double1
  my $w = shift;
  my $Ev = $w->XEvent;
 
- delete $w->{'shiftanchor'};
+ delete $w->{'shiftanchor'}; 
 
- my $ent = $w->GetNearest($Ev->y, 1);
+ my $ent = $w->GetNearest($Ev->y);
 
  return unless (defined($ent) and length($ent));
 
  $w->anchor('set', $ent)
-	unless(defined $w->info('anchor'));
+	unless($w->info('anchor'));
 
  $w->select('set', $ent);
  $w->Callback(-command => $ent);
@@ -382,17 +344,17 @@ sub CtrlButton1
  my $w = shift;
  my $Ev = $w->XEvent;
 
- delete $w->{'shiftanchor'};
+ delete $w->{'shiftanchor'}; 
 
- my $ent = $w->GetNearest($Ev->y, 1);
+ my $ent = $w->GetNearest($Ev->y);
 
  return unless (defined($ent) and length($ent));
 
  my $mode = $w->cget('-selectmode');
 
- if($mode eq 'extended')
+ if($mode eq "extended")
   {
-   $w->anchor('set', $ent) unless( defined $w->info('anchor') );
+   $w->anchor('set', $ent) unless( $w->info('anchor') );
 
    if($w->select('includes', $ent))
     {
@@ -414,15 +376,15 @@ sub UpDown
  my $done = 0;
  my $anchor = $w->info('anchor');
 
- delete $w->{'shiftanchor'};
+ delete $w->{'shiftanchor'}; 
 
- unless( defined $anchor )
+ unless( $anchor )
   {
-   $anchor = ($w->info('children'))[0] || '';
+   $anchor = ($w->info('children'))[0] || "";
 
    return unless (defined($anchor) and length($anchor));
 
-   if($w->entrycget($anchor, '-state') ne 'disabled')
+   if($w->entrycget($anchor, '-state') ne "disabled")
     {
      # That's a good anchor
      $done = 1;
@@ -441,13 +403,13 @@ sub UpDown
  while(!$done)
   {
    $ent = $w->info($spec, $ent);
-   last unless( defined $ent );
-   next if( $w->entrycget($ent, '-state') eq 'disabled' );
+   last unless( $ent );
+   next if( $w->entrycget($ent, '-state') eq "disabled" );
    next if( $w->info('hidden', $ent) );
    last;
   }
 
- unless( defined $ent )
+ unless( $ent )
   {
    $w->yview('scroll', $spec eq 'prev' ? -1 : 1, 'unit');
    return;
@@ -456,7 +418,7 @@ sub UpDown
  $w->anchor('set', $ent);
  $w->see($ent);
 
- if($w->cget('-selectmode') ne 'single')
+ if($w->cget('-selectmode') ne "single")
   {
    $w->select('clear');
    $w->selection('set', $ent);
@@ -472,7 +434,7 @@ sub ShiftUpDown
  my $mode = $w->cget('-selectmode');
 
  return $w->UpDown($spec)
-   if($mode eq 'single' || $mode eq 'browse');
+   if($mode eq "single" || $mode eq "browse");
 
  my $anchor = $w->info('anchor');
 
@@ -480,15 +442,15 @@ sub ShiftUpDown
 
  my $done = 0;
 
- $w->{'shiftanchor'} = $anchor unless( $w->{'shiftanchor'} );
+ $w->{'shiftanchor'} = $anchor unless( $w->{'shiftanchor'} ); 
 
  my $ent = $w->{'shiftanchor'};
 
  while( !$done )
   {
    $ent = $w->info($spec, $ent);
-   last unless( defined $ent );
-   next if( $w->entrycget($ent, '-state') eq 'disabled' );
+   last unless( $ent );
+   next if( $w->entrycget($ent, '-state') eq "disabled" );
    next if( $w->info('hidden', $ent) );
    last;
   }
@@ -503,8 +465,8 @@ sub ShiftUpDown
  $w->selection('set', $anchor, $ent);
  $w->see($ent);
 
- $w->{'shiftanchor'} = $ent;
-
+ $w->{'shiftanchor'} = $ent; 
+ 
  $w->Callback(-browsecmd =>$ent);
 }
 
@@ -513,13 +475,13 @@ sub LeftRight
  my $w = shift;
  my $spec = shift;
 
- delete $w->{'shiftanchor'};
+ delete $w->{'shiftanchor'}; 
 
  my $anchor = $w->info('anchor');
 
- unless(defined $anchor)
+ unless($anchor)
   {
-   $anchor = ($w->info('children'))[0] || '';
+   $anchor = ($w->info('children'))[0] || "";
   }
 
  my $done = 0;
@@ -529,42 +491,42 @@ sub LeftRight
   {
    my $e = $ent;
 
-   if($spec eq 'left')
+   if($spec eq "left")
     {
      $ent = $w->info('parent', $e);
 
      $ent = $w->info('prev', $e)
-       unless(defined $ent && $w->entrycget($ent, '-state') ne 'disabled')
+       unless($ent && $w->entrycget($ent, '-state') ne "disabled")
     }
    else
     {
      $ent = ($w->info('children', $e))[0];
 
      $ent = $w->info('next', $e)
-       unless(defined $ent && $w->entrycget($ent, '-state') ne 'disabled')
+       unless($ent && $w->entrycget($ent, '-state') ne "disabled")
     }
 
-   last unless( defined $ent );
-   last if($w->entrycget($ent, '-state') ne 'disabled');
+   last unless( $ent );
+   last if($w->entrycget($ent, '-state') ne "disabled");
   }
 
- unless( defined $ent )
+ unless( $ent )
   {
-   $w->xview('scroll', $spec eq 'left' ? -1 : 1, 'unit');
+   $w->xview('scroll', $spec eq "left" ? -1 : 1, 'unit');
    return;
   }
 
  $w->anchor('set', $ent);
  $w->see($ent);
 
- if($w->cget('-selectmode') ne 'single')
+ if($w->cget('-selectmode') ne "single")
   {
    $w->select('clear');
    $w->selection('set', $ent);
 
    $w->Callback(-browsecmd =>$ent);
   }
-}
+}   
 
 sub KeyboardHome
 {
@@ -573,13 +535,6 @@ sub KeyboardHome
  $w->xview('moveto' => 0);
 }
 
-sub KeyboardEnd
-{
- my $w = shift;
- $w->yview('moveto' => 1);
- $w->xview('moveto' => 0);
-}
-
 sub KeyboardActivate
 {
  my $w = shift;
@@ -621,8 +576,8 @@ sub AutoScan
 {
  my $w = shift;
 
- return if($w->cget('-selectmode') eq 'dragdrop');
-
+ return if($w->cget('-selectmode') eq "dragdrop");
+ 
  my $Ev = $w->XEvent;
  my $y = $Ev->y;
  my $x = $Ev->x;
@@ -647,7 +602,7 @@ sub AutoScan
   {
    return;
   }
- $w->RepeatId($w->SUPER::after(50,[ AutoScan => $w ]));
+ $w->RepeatId($w->after(50,"AutoScan",$w));
  $w->Button1Motion;
 }
 
