@@ -1,47 +1,49 @@
+package Tk::TixGrid; 
 
-# TODO:
-#
-#	o How to get into state 's0' 'b0' so cursor keys start
-#	  working (compare with Tk/Widget  XYscrollBind
-#	o the options -browsecmd and -command callback are not
-#	  not implemented (as in Tix)
-#	o privateData 'state' used only once (check again Grid.tcl)
-#	o FloatEntry 'sometimes not activeted immediately on selection
-#	o check also Leave Binding. Looks like entry does get unpost'ed
+use vars qw($VERSION @ISA);
+$VERSION = '3.008'; # $Id: //depot/Tk8/TixGrid/TixGrid.pm#8$
 
-package Tk::TixGrid;
-
-BEGIN
-  {
-    use vars '$DEBUG';
-    $DEBUG = (defined($ENV{USER}) and $ENV{USER} eq 'ach') ? 1 : 0;
-    print STDERR "tixGrid: debug = $DEBUG\n" if $DEBUG;
-  }
-
-use strict;
-use vars qw($VERSION);
-$VERSION = '3.022'; # $Id: //depot/Tk8/TixGrid/TixGrid.pm#22 $
-
-use Tk qw(Ev $XS_VERSION);
+use Tk 'Ev';
 use Tk::Widget;
-use Carp;
 
-carp __PACKAGE__ . ' is deprecated';
-
-use base  'Tk::Widget';
+@ISA = 'Tk::Widget';
 
 Construct Tk::Widget 'TixGrid';
 
-bootstrap Tk::TixGrid;
+bootstrap Tk::TixGrid $Tk::VERSION; 
 
 sub Tk_cmd { \&Tk::tixGrid }
 
 sub Tk::Widget::SrclTixGrid { shift->Scrolled('TixGrid' => @_) }
 
-Tk::Methods qw(anchor bdtype delete entrycget entryconfigure format index
-               move set size unset xview yview
-               to_list_commands dragsite dropsite geometryinfo info
-               nearest selection sort );
+EnterMethods Tk::TixGrid __FILE__,qw(
+			anchor
+			bdtype
+			cget
+			configure
+			delete
+			entrycget
+			entryconfigure
+			format
+			index
+			move
+			set
+			size
+			unset
+			xview
+			yview
+			),
+# TODO:  $tixgrid->to_list_commands lists (not documented in tix manpage):
+#
+	'to_list_commands',	# just to get the list :-)
+	'dragsite',
+	'dropsite',
+	'geometryinfo',
+	'info',
+	'nearest',
+	'selection',
+	'sort',
+	;
 
 # edit subcommand is special. It justs invokes tcl code:
 #
@@ -131,12 +133,52 @@ sub ClassInit
     $mw->bind($class, '<Control-ButtonPress-1>',['Control_Button_1',	Ev('x'), Ev('y')]);
     $mw->bind($class, '<ButtonRelease-1>',	['ButtonRelease_1',	Ev('x'), Ev('y')]);
     $mw->bind($class, '<Double-ButtonPress-1>', ['Double_1',		Ev('x'), Ev('y')]);
-    $mw->bind($class, '<B1-Motion>','Button_Motion');
-    $mw->bind($class, '<Control-B1-Motion>','Control_Button_Motion');
-    $mw->bind($class, '<B1-Leave>','Button_Leave');
+    $mw->bind($class, '<B1-Motion>',
+				sub
+				{
+				    my $w = shift;
+				    my $Ev = $w->XEvent;
+				    $Tk::x =  $Ev->x;
+				    $Tk::y =  $Ev->y;
+				    $Tk::X =  $Ev->X;
+				    $Tk::Y =  $Ev->Y;
+				    $w->B1_Motion($Tk::x, $Tk::y);
+    				} );
+    $mw->bind($class, '<Control-B1-Motion>', 
+				sub
+				{
+				    my $w = shift;
+				    my $Ev = $w->XEvent;
+				    $Tk::x =  $Ev->x;
+				    $Tk::y =  $Ev->y;
+				    $Tk::X =  $Ev->X;
+				    $Tk::Y =  $Ev->Y;
+				    $w->Control_B1_Motion($Tk::x, $Tk::y);
+    				} );
+    $mw->bind($class, '<B1-Leave>',
+                                sub 
+                                {   
+                                    my $w = shift;
+                                    my $Ev = $w->XEvent;
+                                    $Tk::x =  $Ev->x;
+                                    $Tk::y =  $Ev->y;
+                                    $Tk::X =  $Ev->X;
+                                    $Tk::Y =  $Ev->Y;
+                                    $w->B1_Leave();
+                                } );
     $mw->bind($class, '<Double-ButtonPress-1>', ['Double_1',		Ev('x'), Ev('y')]);
     $mw->bind($class, '<B1-Enter>',		['B1_Enter',		Ev('x'), Ev('y')]);
-    $mw->bind($class, '<Control-B1-Leave>','Control_Button_Leave');
+    $mw->bind($class, '<Control-B1-Leave>',
+                                sub
+                                {
+                                    my $w = shift;
+                                    my $Ev = $w->XEvent;
+                                    $Tk::x =  $Ev->x;
+                                    $Tk::y =  $Ev->y;
+                                    $Tk::X =  $Ev->X;
+                                    $Tk::Y =  $Ev->Y;
+                                    $w->Control_B1_Leave();
+                                } );
     $mw->bind($class, '<Control-B1-Enter>',	['Control_B1_Enter',	Ev('x'), Ev('y')]);
 
     ##
@@ -153,7 +195,7 @@ sub ClassInit
     $mw->bind($class, '<Return>',	'Return');
     $mw->bind($class, '<space>',	'Space'	);
 
-    return $class;
+    return $class; 
   }
 
 #----------------------------------------------------------------------
@@ -165,11 +207,11 @@ sub ClassInit
 #----------------------------------------------------------------------
 
 sub Button_1
-  {
+  { 
     my $w = shift;
 
     return if $w->cget('-state') eq 'disabled';
-    $w->SetFocus;
+    $w->SetFocus; 
     $w->ChgState(@_,
 		[
 		'0'=>'1',
@@ -182,20 +224,20 @@ sub Shift_Button_1
     my $w = shift;
 
     return if $w->cget('-state') eq 'disabled';
-    $w->SetFocus;
+    $w->SetFocus; 
 
 #    $w->ChgState(@_,
 #		[
 #		]
 #		);
-  }
+  } 
 
 sub Control_Button_1
   {
     my $w = shift;
 
     return if $w->cget('-state') eq 'disabled';
-    $w->SetFocus;
+    $w->SetFocus; 
 
     $w->ChgState(@_,
 		[
@@ -205,7 +247,7 @@ sub Control_Button_1
 		'e0'	=> 'e10',
 		]
 	 	);
-  }
+  } 
 
 sub ButtonRelease_1
   {
@@ -215,7 +257,7 @@ sub ButtonRelease_1
 		'4'	=> '3',
 		]
 		);
-  }
+  } 
 
 sub B1_Motion
   {
@@ -225,7 +267,7 @@ sub B1_Motion
 		'4'	=> '4',
 		]
 		);
-  }
+  } 
 
 
 sub Control_B1_Motion
@@ -240,7 +282,7 @@ sub Control_B1_Motion
 		'm5'	=> 'm4',
 		]
 		);
-  }
+  } 
 
 
 sub Double_1
@@ -251,7 +293,7 @@ sub Double_1
 		'b0'	=> 'b7',
 		]
 		);
-  }
+  } 
 
 
 sub B1_Leave
@@ -268,7 +310,7 @@ sub B1_Leave
 		'e5'	=> 'e8',
 		]
 		);
-  }
+  } 
 
 
 sub B1_Enter
@@ -285,7 +327,7 @@ sub B1_Enter
 		'e9'	=> 'e4',
 		]
 		);
-  }
+  } 
 
 
 sub Control_B1_Leave
@@ -300,7 +342,7 @@ sub Control_B1_Leave
 		'm5'	=> 'm8',
 		]
 		);
-  }
+  } 
 
 
 sub Control_B1_Enter
@@ -315,7 +357,7 @@ sub Control_B1_Enter
 		'm9'	=> 'm4',
 		]
 		);
-  }
+  } 
 
 
 sub AutoScan
@@ -332,7 +374,7 @@ sub AutoScan
 		'e9'	=> 'm9',
 		]
 		);
-  }
+  } 
 
 #----------------------------------------------------------------------
 #
@@ -348,44 +390,44 @@ sub DirKey
 
     return if $w->cget('-state') eq 'disabled';
 
-print STDERR "$w->DirKey($key)\n" if $DEBUG;
+print "$w->DirKey($key)\n";
     $w->ChgState($key,
 		[
 		's0'	=> 's8',
 		'b0'	=> 'b8',
 		]
 		);
-  }
+  } 
 
 
 sub Return
   {
-    my ($w) = @_;
+    my ($w, $key) = @_;
 
     return if $w->cget('-state') eq 'disabled';
 
-    $w->ChgState(
+    $w->ChgState($key,
 		[
 		's0'	=> 's9',
 		'b0'	=> 'b9',
 		]
 		);
-  }
+  } 
 
 
 sub Space
   {
-    my ($w) = @_;
+    my ($w, $key) = @_;
 
     return if $w->cget('-state') eq 'disabled';
 
-    $w->ChgState(
+    $w->ChgState($key,
 		[
 		's0'	=> 's10',
 		'b0'	=> 'b10',
 		]
 		);
-  }
+  } 
 
 
 #----------------------------------------------------------------------
@@ -395,7 +437,7 @@ sub Space
 #
 #----------------------------------------------------------------------
 
-sub GetState
+sub GetState 
   {
     my ($w) = @_;
     my $data = $w->privateData();
@@ -403,54 +445,6 @@ sub GetState
     return $data->{state};
 }
 
-sub Button_Motion
-{
- my $w = shift;
- my $Ev = $w->XEvent;
- $Tk::x =  $Ev->x;
- $Tk::y =  $Ev->y;
- $Tk::X =  $Ev->X;
- $Tk::Y =  $Ev->Y;
- $w->B1_Motion($Tk::x, $Tk::y);
-}
-
-
-sub Control_Button_Motion
-{
- my $w = shift;
- my $Ev = $w->XEvent;
- $Tk::x =  $Ev->x;
- $Tk::y =  $Ev->y;
- $Tk::X =  $Ev->X;
- $Tk::Y =  $Ev->Y;
- $w->Control_B1_Motion($Tk::x, $Tk::y);
-}
-
-
-sub Button_Leave
-{
- my $w = shift;
- my $Ev = $w->XEvent;
- $Tk::x =  $Ev->x;
- $Tk::y =  $Ev->y;
- $Tk::X =  $Ev->X;
- $Tk::Y =  $Ev->Y;
- $w->B1_Leave();
-}
-
-
-sub Control_Button_Leave
-{
- my $w = shift;
- my $Ev = $w->XEvent;
- $Tk::x =  $Ev->x;
- $Tk::y =  $Ev->y;
- $Tk::X =  $Ev->X;
- $Tk::Y =  $Ev->Y;
- $w->Control_B1_Leave();
-}
-
-
 sub SetState
   {
     my ($w, $state) = @_;
@@ -460,24 +454,21 @@ sub SetState
 sub GoState
   {
     my ($w, $state) = (shift, shift);
-    print STDERR 'Gostate:  ', $w->GetState, " --> $state, " if $DEBUG;
-    $w->SetState($state);
+print STDERR "Gostate: '", $w->GetState, "' --> '$state'\n";
+    $w->SetState($state); 
     my $method = "GoState_$state";
 
-    print STDERR 'args=(', join(',',@_), ')'.
-	"\t(",$w->cget('-selectmode').
-	',',$w->cget('-selectunit').")\n" if $DEBUG;
-
     if (0)
       {
-	$@ = '';
+	$@ = ''; 
 	%@ = (); 		# Workaround to prevent spurious loss of $@
 	eval { $w->$method(@_) };
 	print STDERR "Error Gostate: '$state': ", $@ if $@;
 	return undef;
       }
-
-    $w->$method(@_);
+print "$method(", join(',',@_), ")\n";
+    #eval { $w->$method(@_); };
+    eval { $w->$method(@_); };
     return undef
   }
 
@@ -489,25 +480,28 @@ sub ChgState
   {
     my $w   = shift;
     my $map = pop;
-    print STDERR 'ChgState(', join(',',@_,'['), join(',',@$map,),'])  ' if $DEBUG;
+print "ChgState(", join(',',@_,'['), join(",",@$map,),"])\n";
     my $state = $w->GetState;
-
     my ($match, $to);
+my @tmp = @$map;
     while (@$map)
       {
-        $match = shift @$map;
-        $to    = shift @$map;
+        $match = shift @$map; 
+        $to    = shift @$map; 
+# 3rd try:
+        #if ($match =~ /$state/) 
         if ($match eq $state)
           {
-	    print STDERR "$state --> $to \n" if $DEBUG;
+print STDERR "Chg: $state  ===>>  $to \n";
 	    $w->GoState($to, @_);
 	    return;
 	  }
       }
-    print STDERR "*no* chg for $state\n" if $DEBUG;
+print STDERR join('|', "Chg no new state for '$state' in ", @tmp, "\n");
   }
 
 
+
 #----------------------------------------------------------------------
 #		   SELECTION ROUTINES
 #----------------------------------------------------------------------
@@ -519,10 +513,9 @@ sub ChgState
 
 sub SelectSingle
   {
-    my ($w, $n1, $n2) = @_;
-    $w->selection('set', $n1, $n2);
-    #FIX: -options -browsecmd not implemented jet
-    #$w->Callback('-browsecmd' => $n1, $n2);
+    my ($w, $ent) = @_;
+    $w->selectionSet($ent->[0], $ent->[1]);
+    #$w->Call('browsecmd', @$ent);
   }
 
 #----------------------------------------------------------------------
@@ -537,12 +530,11 @@ sub GoState_0
 
     foreach my $cmd (@$list)
       {
-        # XXX should do more something like $w->Callback'('__pending_cmds__');
 	eval $cmd;		# XXX why in tcl in global context (binding?)
       }
     undef(@$list); 		# XXX should really delete? Maybe on needed in TCL
   }
-
+ 
 # XXXX how to translate global context
 #      what does unset
 #proc tixGrid:GoState-0 {w} {
@@ -559,6 +551,7 @@ sub GoState_0
 #    }
 #}
 
+
 sub GoState_1
    {
      my ($w, $x, $y) = @_;
@@ -569,19 +562,18 @@ sub GoState_1
          $w->SetAnchor(@ent);
        }
      $w->CheckEdit;
-     $w->selection('clear', 0, 0, 'max', 'max');
+     $w->selectionClear(0, 0, 'max', 'max');
 
-     if ($w->cget('-selectmode') ne 'single')
+     if ($w->cget('-selectmode') eq 'single')
        {
          $w->SelectSingle(@ent);
        }
-     $w->GoState(2);
+     $w->GoState_2
    }
 
-sub GoState_2
-  {
-    my ($w) = @_;
-  }
+
+sub GoState_2 { }
+     
 
 sub GoState_3
    {
@@ -592,10 +584,51 @@ sub GoState_3
        {
          $w->SelectSingle(@ent);
        }
-     $w->GoState(0);
+     $w->GoState_0;
 
    }
 
+
+sub GoState_5
+   {
+     my ($w, $x, $y) = @_;
+
+     my @ent = $w->mynearest($x,$y);
+     if (@ent)
+       {
+         $w->SelectSingle(@ent);
+         $w->SetEdit(@ent);
+       }
+     $w->GoState_0;
+
+   }
+
+##############################################
+# BUG xxx
+#	return scalar instead of errors
+
+sub mynearest   { shift->split_s2a('nearest', @_); }
+sub myanchorGet { shift->split_s2a('anchorGet', @_); }
+
+my $DEBUG_SPLIT_S2A = 1;
+
+sub split_s2a
+  {
+    my $w = shift;
+    my $method = shift;
+    my @ent = $w->$method(@_);
+    if (@ent == 1)
+      {
+my $tmp = $ent[0];
+        @ent = split(/ /, $ent[0]) if @ent == 1;
+print STDERR join("|","$method splitted '$tmp' =>",@ent,"\n") if $DEBUG_SPLIT_S2A;
+      }
+    return @ent;
+  }
+
+##############################################
+
+
 sub GoState_4
   {
     my ($w, $x, $y) = @_;
@@ -610,68 +643,27 @@ sub GoState_4
     elsif ($mode eq 'browse')
       {
 	$w->SetAnchor(@ent);
-	$w->selection('clear', 0, 0, 'max', 'max');
+	$w->selectionClear(0, 0, 'max', 'max');
 	$w->SelectSingle(@ent);
       }
     elsif ($mode eq 'multiple' ||
 	   $mode eq 'extended')
       {
-	my (@anchor) = $w->anchor('get');
-	$w->selection('adjust', @anchor[0,1], @ent[0,1]);
+	my (@anchor) = $w->anchorGet;
+	$w->selectionAdjust(@anchor[0,1], @ent[0,1]);
       }
   }
 
-sub GoState_5
-   {
-     my ($w, $x, $y) = @_;
-
-     my @ent = $w->mynearest($x,$y);
-     if (@ent)
-       {
-         $w->SelectSingle(@ent);
-         $w->SetEdit(@ent);
-       }
-     $w->GoState(0);
-
-   }
-
-##############################################
-# BUG xxx
-#	return scalar instead of errors
-
-sub mynearest   { shift->split_s2a('nearest', @_); }
-sub myanchorGet { shift->split_s2a('anchor', 'get', @_); }
-
-sub split_s2a
-  {
-    my $w = shift;
-    my $method = shift;
-    my @ent = $w->$method(@_);
-    if (@ent == 1)
-      {
-my $tmp = $ent[0];
-        @ent = split(/ /, $ent[0]) if @ent == 1;
-print STDERR join('|',"$method splitted '$tmp' =>",@ent,"\n") if $DEBUG;
-      }
-    else
-      {
-#print STDERR join("|","$method splitted is okay :",@ent,"\n") if $DEBUG;
-      }
-    return @ent;
-  }
-
-##############################################
-
 
 sub GoState_s5
   {
-    shift->StartScan();
+    shift->StartScan;
   }
 
 
 sub GoState_s6
   {
-    shift->DoScan();
+    shift->DoScan;
   }
 
 
@@ -682,23 +674,25 @@ sub GoState_s7
     my @ent = $w->mynearest($x, $y);
     if (@ent)
       {
-        $w->selection('clear');
-	$w->selection('set', @ent);
-	$w->Callback('-command' => @ent);
+        $w->selectionClear;
+	$w->selectionSet(@ent);
+	$w->Call('command', @ent);
       }
-    $w->GoState('s0');
+    $w->GoState_s0;
   }
 
 
 sub GoState_s8
   {
     my ($w, $key) = @_;
-
+    
     ## BUGS ....
     ## - anchor is bad, only bbox, exists8
     ## - looks like anchor is 1-dim: set anchor 0
     ## - method see unknown  (even when defined with Tk::Method)
 
+#print  "GoState_s8 is not implemented\n";
+
     my (@anchor) = $w->info('anchor');
     if (@anchor)
       {
@@ -708,11 +702,11 @@ sub GoState_s8
       {
         @anchor = $w->info($key, @anchor);
       }
-
-    $w->anchor('set', @anchor);
+ 
+    $w->anchorSet(@anchor);
     $w->see(@anchor);
-
-    $w->GoState('s0');
+ 
+    $w->GoState_s0  
   }
 
 #proc tixGrid:GoState-s8 {w key} {
@@ -731,54 +725,54 @@ sub GoState_s8
 
 
 sub GoState_s9
-  {
+  { 
     my ($w, $key) = @_;
 
-#print STDERR "GoState_s9 is not implemented\n";
+#print  "GoState_s9 is not implemented\n";
 
     my (@anchor) = $w->info('anchor');
     unless (@anchor)
       {
         @anchor = ();
-        $w->anchor('set', @anchor);
+        $w->anchorSet(@anchor);
         $w->see(@anchor);
       }
-
+ 
     unless ($w->info('anchor'))
       {
         # ! may not have any elements
         #
-        $w->Callback('-command' => $w->info('anchor'));
-        $w->selection('clear');
-        $w->selection('set', @anchor);
+        $w->Call('command', $w->info('anchor'));
+        $w->selectionClear;
+        $w->selectionSet(@anchor);
       }
-
-      $w->GoState('s0');
+ 
+      $w->GoState_s0;
   }
 
 
 sub GoState_s10
-  {
+  { 
     my ($w, $key) = @_;
 
     my (@anchor) = $w->info('anchor');
     if (@anchor)
       {
         @anchor = ();
-        $w->anchor('set', @anchor);
+        $w->anchorSet(@anchor);
         $w->see(@anchor);
       }
-
+ 
     unless ($w->info('anchor'))
       {
         # ! may not have any elements
         #
-        $w->Callback('-browsecmd' => $w->info('anchor'));
-        $w->selection('clear');
-        $w->selection('set', @anchor);
+        $w->Call('browsecmd', $w->info('anchor'));
+        $w->selectionClear;
+        $w->selectionSet(@anchor);
       }
-
-    $w->GoState('s0');
+ 
+    $w->GoState_s0;
   }
 
 
@@ -786,10 +780,7 @@ sub GoState_s10
 #	BROWSE SELECTION
 #----------------------------------------------------------------------
 
-sub GoState_b0
-  {
-    my ($w) = @_;
-  }
+sub GoState_b0 { }
 
 sub GoState_b1
   {
@@ -798,19 +789,16 @@ sub GoState_b1
     my (@ent) = $w->mynearest($x, $y);
     if (@ent)
       {
-	$w->anchor('set', @ent);
-	$w->selection('clear');
-	$w->selection('set', @ent);
-	$w->Callback('-browsecmd' => @ent);
+	$w->anchorSet(@ent);
+	$w->selectionClear;
+	$w->selectionSet(@ent);
+	$w->Call('browsecmd', @ent);
       }
 
-    $w->GoState('b2');
+    $w->GoState_b2
   }
 
-sub GoState_b2
-  {
-    my ($w) = @_;
-  }
+sub GoState_b2 { }
 
 sub GoState_b3
   {
@@ -819,13 +807,13 @@ sub GoState_b3
     my (@ent) = $w->info('anchor');
     if (@ent)
       {
-	$w->selection('clear');
-	$w->selection('set', @ent);
-	$w->selection('set', @ent);
-	$w->Callback('-browsecmd' => @ent);
+	$w->selectionClear;
+	$w->selectionSet(@ent);
+	$w->selectionSet(@ent);
+	$w->Call('browsecmd', @ent);
       }
 
-    $w->GoState('b0');
+    $w->GoState_b0
   }
 
 
@@ -836,18 +824,18 @@ sub GoState_b4
     my (@ent) = $w->mynearest($x, $y);
     if (@ent)
       {
-	$w->anchor('set', @ent);
-	$w->selection('clear');
-	$w->selection('set', @ent);
-	$w->Callback('-browsecmd' => @ent);
+	$w->anchorSet(@ent);
+	$w->selectionClear;
+	$w->selectionSet(@ent);
+	$w->Call('browsecmd', @ent);
       }
   }
 
 
-sub GoState_b5 { shift->StartScan(); }
+sub GoState_b5 { shift->StartScan }
 
 
-sub GoState_b6 { shift->DoScan(); }
+sub GoState_b6 { shift->DoScan }
 
 
 sub GoState_b7
@@ -857,11 +845,10 @@ sub GoState_b7
      my (@ent) = $w->mynearest($x, $y);
      if (@ent)
        {
-         $w->selection('clear');
-	 $w->selection('set', @ent);
-         $w->Callback('-command' => @ent);
+         $w->selectionClear;
+	 $w->selectionSet(@ent);
+         $w->Call('command', @ent);
        }
-     $w->GoState('b0');
   }
 
 
@@ -878,14 +865,14 @@ sub GoState_b8
       {
         @anchor = (0,0);   # ?????
       }
-
-    $w->anchor('set', @anchor);
-    $w->selection('clear');
-    $w->selection('set', @anchor);
+ 
+    $w->anchorSet(@anchor);
+    $w->selectionClear;
+    $w->selectionSet(@anchor);
     $w->see(@anchor);
-
-    $w->Callback('-browsecmd' => @anchor);
-    $w->GoState('b0');
+ 
+    $w->Call('browsecmd', @anchor);
+    $w->GoState_b0;
   }
 
 
@@ -897,7 +884,7 @@ sub GoState_b9
     unless (@anchor)
       {
 	@anchor = (0,0);
-        $w->anchor('set', @anchor);
+        $w->anchorSet(@anchor);
 	$w->see(@anchor);
       }
 
@@ -905,12 +892,12 @@ sub GoState_b9
       {
 	# ! may not have any elements
 	#
-	$w->Callback('-command' => $w->info('anchor'));
-	$w->selection('clear');
-	$w->selection('set', @anchor);
+	$w->Call('command', $w->info('anchor'));
+	$w->selectionClear;
+	$w->selectionSet(@anchor);
       }
 
-    $w->GoState('b0');
+    $w->GoState_b0;
   }
 
 
@@ -922,7 +909,7 @@ sub GoState_b10
     unless (@anchor)
       {
 	@anchor = (0,0);
-        $w->anchor('set', @anchor);
+        $w->anchorSet(@anchor);
 	$w->see(@anchor);
       }
 
@@ -930,12 +917,12 @@ sub GoState_b10
       {
 	# ! may not have any elements
 	#
-	$w->Callback('-browsecmd' => $w->info('anchor'));
-	$w->selection('clear');
-	$w->selection('set', @anchor);
+	$w->Call('browsecmd', $w->info('anchor'));
+	$w->selectionClear;
+	$w->selectionSet(@anchor);
       }
 
-    $w->GoState('b0');
+    $w->GoState_b0;
   }
 
 #----------------------------------------------------------------------
@@ -943,10 +930,8 @@ sub GoState_b10
 #----------------------------------------------------------------------
 
 
-sub GoState_m0
-  {
-    my ($w) = @_;
-  }
+sub GoState_m0 { }
+
 
 sub GoState_m1
   {
@@ -955,19 +940,17 @@ sub GoState_m1
     my (@ent) = $w->mynearest($x,$y);
     if (@ent)
       {
-	$w->anchor('set', @ent);
-	$w->selection('clear');
-	$w->selection('set', @ent);
-	$w->Callback('-browsecmd' => @ent);
+	$w->anchorSet(@ent);
+	$w->selectionClear;
+	$w->selectionSet(@ent);
+	$w->Call('browsecmd', @ent);
       }
 
-    $w->GoState('m2');
+    $w->GoState_m2;
   }
 
-sub GoState_m2
-  {
-    my ($w) = @_;
-  }
+
+sub GoState_m2 { }
 
 sub GoState_m3
   {
@@ -976,10 +959,10 @@ sub GoState_m3
     my (@ent) = $w->info('anchor');
     if (@ent)
       {
-	$w->Callback('-browsecmd' => @ent);
+	$w->Call('browsecmd', @ent);
       }
 
-    $w->GoState('m0');
+    $w->GoState_m0;
   }
 
 
@@ -991,17 +974,16 @@ sub GoState_m4
     my (@to)   = $w->mynearest($x, $y);
     if (@to)
       {
-	$w->selection('clear');
-	$w->selection('set', @from, @to);
-	$w->Callback('-browsecmd' => @to);
+	$w->selectionClear;
+	$w->selectionSet(@from, @to);
+	$w->Call('browsecmd', @to);
       }
-    $w->GoState('m5');
+
+    $w->GoState_m5;
   }
 
-sub GoState_m5
-  {
-    my ($w) = @_;
-  }
+sub GoState_m5 { }
+
 
 sub GoState_m6
   {
@@ -1010,9 +992,10 @@ sub GoState_m6
     my (@ent)   = $w->mynearest($x, $y);
     if (@ent)
       {
-	$w->Callback('-browsecmd' => @ent);
+	$w->Call('browsecmd', @ent);
       }
-    $w->GoState('m0');
+
+    $w->GoState_m0;
   }
 
 sub GoState_m7
@@ -1024,22 +1007,23 @@ sub GoState_m7
     unless (@from)
       {
 	@from = @to;
-	$w->anchor('set', @from);
+	$w->anchorSet(@from);
       }
     if (@to)
       {
-	$w->selection('clear');
-	$w->selection('set', @from, @to);
-	$w->Callback('-browsecmd' => @to);
+	$w->selectionClear;
+	$w->selectionSet(@from, @to);
+	$w->Call('browsecmd', @to);
       }
-    $w->GoState('m5');
+
+    $w->GoState_m5;
   }
 
+   
+sub GoState_m8 { shift->StartScan }
 
-sub GoState_m8 { shift->StartScan() }
-
-
-sub GoState_m9 { shift->DoScan() }
+   
+sub GoState_m9 { shift->DoScan }
 
 
 sub GoState_xm7
@@ -1049,21 +1033,20 @@ sub GoState_xm7
     my (@ent)   = $w->mynearest($x, $y);
     if (@ent)
       {
-	$w->selection('clear');
-	$w->selection('set', @ent);
-	$w->Callback('-browsecmd' => @ent);
+	$w->selectionClear;
+	$w->selectionSet(@ent);
+	$w->Call('browsecmd', @ent);
       }
-    $w->GoState('m0');
+
+    $w->GoState_m0;
   }
 
+
 #----------------------------------------------------------------------
 #	EXTENDED SELECTION
 #----------------------------------------------------------------------
 
-sub GoState_e0
-  {
-    my ($w) = @_;
-  }
+sub GoState_e0 { }
 
 sub GoState_e1
   {
@@ -1071,19 +1054,16 @@ sub GoState_e1
     my (@ent) = $w->mynearest($x, $y);
     if (@ent)
       {
-	$w->anchor('set', @ent);
-	$w->selection('clear');
-	$w->selection('set', @ent);
-	$w->Callback('-browsecmd' => @ent);
+	$w->anchorSet(@ent);
+	$w->selectionClear;
+	$w->selectionSet(@ent);
+	$w->Call('browsecmd', @ent);
       }
-    $w->GoState('e2');
+    $w->GoState_e2;
   }
 
 
-sub GoState_e2
-  {
-    my ($w) = @_;
-  }
+sub GoState_e2 { }
 
 sub GoState_e3
   {
@@ -1092,30 +1072,30 @@ sub GoState_e3
     my (@ent) = $w->info('anchor');
     if (@ent)
       {
-        $w->Callback('-browsecmd' => @ent);
+        $w->Call('browsecmd', @ent);
       }
-    $w->GoState('e0');
+
+    $w->GoState_e0
   }
 
+
 sub GoState_e4
   {
     my ($w, $x, $y) = @_;
-
     my (@from) = $w->info('anchor');
     my (@to)   = $w->mynearest($x, $y);
     if (@to)
       {
-        $w->selection('clear');
-        $w->selection('set', @from, @to);
-        $w->Callback('-browsecmd' => @to);
+        $w->selectionClear;
+        $w->selectionSet(@from, @to);
+        $w->Call('browsecmd', @to);
       }
-    $w->GoState('e5');
+    $w->GoState_e5;
   }
 
-sub GoState_e5
-  {
-    my ($w) = @_;
-  }
+
+sub GoState_e5 { }
+
 
 sub GoState_e6
   {
@@ -1124,10 +1104,10 @@ sub GoState_e6
     my (@ent)   = $w->mynearest($x, $y);
     if (@ent)
       {
-        $w->Callback('-browsecmd' => @ent);
+        $w->Call('browsecmd', @ent);
       }
-    $w->GoState('e0');
-  }
+    $w->GoState_e0;
+  } 
 
 sub GoState_e7
   {
@@ -1138,57 +1118,61 @@ sub GoState_e7
     unless (@from)
       {
         @from = @to;
-        $w->anchor('set', @from);
+        $w->anchorSet(@from);
       }
     if (@to)
       {
-        $w->selection('clear');
-        $w->selection('set', @from, @to);
-        $w->Callback('-browsecmd' => @to);
+        $w->selectionClear;
+        $w->selectionSet(@from, @to);
+        $w->Call('browsecmd', @to);
       }
-    $w->GoState('e5');
-  }
+    $w->GoState_e5;
+  } 
 
-sub GoState_e8 { shift->StartScan(); }
 
-sub GoState_e9 { shift->DoScan(); }
+sub GoState_e8 { }
+
+
+sub GoState_e9 { shift->DoScan }
+
 
 sub GoState_e10
+  {
+    my ($w, $x, $y) = @_;
+    my (@ent)   = $w->mynearest($x, $y);
+    if (@ent)
+      {
+	if ($w->info('anchor'))
+	  {
+	    $w->anchorSet(@ent);
+	  }
+	if ($w->selectionIncludes(@ent))
+	  {
+	    $w->selectionClear(@ent);
+	  }
+	else
+	  {
+	    $w->selectionSet(@ent);
+	  }
+	$w->Call('browsecmd', @ent);
+      }
+    $w->GoState_e2;
+  }
+
+
+sub GoState_xe7
   {
     my ($w, $x, $y) = @_;
 
     my (@ent)   = $w->mynearest($x, $y);
     if (@ent)
       {
-	if ($w->info('anchor'))
-	  {
-	    $w->anchor('set', @ent);
-	  }
-	if ($w->selection('includes', @ent))
-	  {
-	    $w->selection('clear', @ent);
-	  }
-	else
-	  {
-	    $w->selection('set', @ent);
-	  }
-	$w->Callback('-browsecmd' => @ent);
+        $w->selectionClear;
+        $w->selectionSet(@ent);
+        $w->Call('command', @ent);
       }
-    $w->GoState('e2');
-  }
 
-sub GoState_xe7
-  {
-    my ($w, $x, $y) = @_;
-
-    my (@ent) = $w->mynearest($x, $y);
-    if (@ent)
-      {
-        $w->selection('clear');
-        $w->selection('set', @ent);
-        $w->Callback('-command' => @ent);
-      }
-    $w->GoState('e0');
+    $w->GoState_e0;
   }
 
 
@@ -1199,25 +1183,25 @@ sub GoState_xe7
 sub GoState_12
   {
     my ($w, $x, $y) = @_;
-
     $w->CancelRepeat;		# xxx  will not work
-    $w->GoState(5, $x, $y);
+    $w->GoState_5($x, $y);
   }
 #proc tixGrid:GoState-12 {w x y} {
 #    tkCancelRepeat
-#    tixGrid:GoState 5 $w $x $y
+#    tixGrid:GoState 5 $w $x $y 
 #}
 
 sub GoState_13
   {
-    # FIX:  a) $ent or @ent, b) 13 is never called!!? same in Grid.tcl
-    my ($w, @ent, @oldEnt) = @_;
-
+    my ($w, $ent, $oldEnt) = @_;
     my $data = $w->MainWindow->privateData('Tix');
-    $data->{indicator} = \@ent;
-    $data->{oldEntry}  = \@oldEnt;
-    $w->IndicatorCmd('<Arm>', @ent);
+    $data->{indicator} = $ent;
+    $data->{oldEntry}  = $oldEnt;
+    $w->IndicatorCmd('<Arm>', @$ent);
   }
+#proc tixGrid:GoState-13 {w ent oldEnt} {
+#    global tkPriv
+#    set tkPriv(tix,indicator) $ent
 #    set tkPriv(tix,oldEnt)    $oldEnt
 #    tixGrid:IndicatorCmd $w <Arm> $ent
 #}
@@ -1225,13 +1209,12 @@ sub GoState_13
 sub GoState_14
   {
     my ($w, $x, $y) = @_;
-
     my $data = $w->MainWindow->privateData('Tix');
     if ($w->InsideArmedIndicator($x, $y))
       {
-	$w->anchor('set', @{ $data->{indicator} });
-	$w->selection('clear');
-	$w->selection('set', @{ $data->{indicator} });
+	$w->anchorSet(@{ $data->{indicator} });
+	$w->selectionClear;
+	$w->selectionSet(@{ $data->{indicator} });
 	$w->IndicatorCmd('<Activate>', @{ $data->{indicator} });
       }
     else
@@ -1239,33 +1222,30 @@ sub GoState_14
 	$w->IndicatorCmd('<Disarm>', @{ $data->{indicator} });
       }
     delete($data->{indicator});
-    $w->GoState(0);
+    $w->GoState_0;
   }
 
 sub GoState_16
   {
-    my ($w, @ent) = @_;
-
-    return unless (@ent);
+    my ($w, $ent) = @_;
+    return unless (@$ent);
     if ($w->cget('-selectmode') ne 'single')
       {
-	$w->Select(@ent);
-	$w->Browse(@ent);
+	$w->Select(@$ent);
+	$w->Browse(@$ent);
       }
   }
 
 sub GoState_18
   {
     my ($w) = @_;
-
     $w->CancelRepeat;	## xxx
-    $w->GoState(6, $Tk::x, $Tk::y);
+    $w->GoState_6($w, $Tk::x, $Tk::y);
   }
 
 sub GoState_20
   {
     my ($w, $x, $y) = @_;
-
     my $data = $w->MainWindow->privateData('Tix');
     if ($w->InsideArmedIndicator($x, $y))
       {
@@ -1273,14 +1253,13 @@ sub GoState_20
       }
     else
       {
-	$w->GoState(21, $x, $y);
+	$w->GoState_21($x, $y);
       }
   }
 
 sub GoState_21
   {
     my ($w, $x, $y) = @_;
-
     my $data = $w->MainWindow->privateData('Tix');
     unless ($w->InsideArmedIndicator($x, $y))
       {
@@ -1288,23 +1267,24 @@ sub GoState_21
       }
     else
       {
-	$w->GoState(20, $x, $y);
+	$w->GoState_20($x, $y);
       }
   }
 
+
 sub GoState_22
   {
     my ($w) = @_;
     my $data = $w->MainWindow->privateData('Tix');
     if (@{ $data->{oldEntry} })
       {
-	$w->anchor('set', @{ $data->{oldEntry} });
+	$w->anchorSet(@{ $data->{oldEntry} });
       }
     else
       {
-	$w->anchor('clear');
+	$w->anchorClear;
       }
-    $w->GoState(0);
+    $w->GoState_0;
   }
 
 
@@ -1312,13 +1292,13 @@ sub GoState_22
 #			callback actions
 #----------------------------------------------------------------------
 
+# xxx check @ent of @$ent
 sub SetAnchor
   {
     my ($w, @ent) = @_;
-
     if (@ent)
       {
-	$w->anchor('set', @ent);
+	$w->anchorSet(@ent);
 #	$w->see(@ent);
       }
   }
@@ -1327,8 +1307,8 @@ sub SetAnchor
 sub Select
   {
     my ($w, @ent) = @_;
-    $w->selection('clear');
-    $w->selection('set', @ent)
+    $w->selectionClear;
+    $w->selectionSet(@ent)
   }
 
 # xxx check new After handling
@@ -1406,13 +1386,13 @@ sub EditCell
   {
     my ($w, $x, $y) = @_;
     my $list = $w->privateData()->{'list'};
-    if ($w->GetState == 0)
+    if ($w->GetState eq 0)
       {
 	$w->SetEdit($x, $y);	# xxx really correct ? once 2, once 4 args?
       }
     else
       {
-	push(@$list, [ $w, 'SetEdit', $x, $y]);
+	push(@$list, $w->SetEdit($x, $y));
       }
   }
 #proc tixGrid:EditCell {w x y} {
@@ -1501,6 +1481,8 @@ $w->focus;
       }
   }
 
+1;
+__END__
 
 # tixGrid:SetEdit --
 #
@@ -1509,89 +1491,86 @@ $w->focus;
 
 sub SetEdit
   {
-    my ($w, $px, $py) = @_;
-
+    my ($w, $ent) = @_;		# xxx @$ent or @ent ???
     $w->CheckEdit;
 
-    my $efc = $w->cget('-editnotifycmd');
-    return unless ( defined($efc) && length($efc) );
+proc tixGrid:SetEdit {w ent} {
+    set edit $w.tixpriv__edit
+    tixGrid:CheckEdit $w
 
-    unless ($w->Callback('-editnotifycmd' => $px, $py))
-      {
-	print STDERR "editnotifycmd not defined or returned false\n";
-        return;
-      }
+    set editnotifycmd [$w cget -editnotifycmd]
+    if [tixStrEq $editnotifycmd ""] {
+	return
+    }
+    set px [lindex $ent 0]
+    set py [lindex $ent 1]
 
-    my $oldvalue;
-    if ($w->info('exists', $px, $py))
-      {
-	# if entry doesn't support -text option. Can't edit it.
-	#
-	# If the application wants to force editing of an entry, it could
-	# delete or replace the entry in the editnotifyCmd procedure.
-	#
-	Tk::catch { $oldvalue = $w->entrycget($px, $py, '-text'); };
-        if ($@)
-          {
-	    return;
-	  }
-      }
-    else
-      {
-	$oldvalue = '';
-      }
+    if ![uplevel #0 $editnotifycmd $px $py] {
+	return
+    }
+    if [$w info exists $px $py] {
+	if [catch {
+	    set oldValue [$w entrycget $px $py -text]
+	}] {
+	    # The entry doesn't support -text option. Can't edit it.
+	    #
+	    # If the application wants to force editing of an entry, it could
+	    # delete or replace the entry in the editnotifyCmd procedure.
+	    #
+	    return
+	}
+    } else {
+	set oldValue ""
+    }
 
-    my @bbox = $w->info('bbox', $px, $py);
+    set bbox [$w info bbox [lindex $ent 0] [lindex $ent 1]]
+    set x [lindex $bbox 0]
+    set y [lindex $bbox 1]
+    set W [lindex $bbox 2]
+    set H [lindex $bbox 3]
 
-    my $edit = $w->privateData()->{__EDIT__};
-    unless (Tk::Exists($edit))
-      {
-        require Tk::FloatEntry;
-        $edit = $w->FloatEntry();
-	$w->privateData()->{__EDIT__} = $edit;
-      }
-    $edit->configure(-command=>[\&DoneEdit, $w,  $px, $py]);
-    $edit->post(@bbox);
-    $edit->configure(-value=>$oldvalue);
+    if ![winfo exists $edit] {
+	tixFloatEntry $edit
+    }
+
+    $edit config -command "tixGrid:DoneEdit $w $ent"
+    $edit post $x $y $W $H
+
+    $edit config -value $oldValue
+}
+
+proc tixGrid:DoneEdit {w x y args} {
+    set edit $w.tixpriv__edit
+    $edit config -command ""
+    $edit unpost
+
+    set value [tixEvent value]
+    if [$w info exists $x $y] {
+	if [catch {
+	    $w entryconfig $x $y -text $value
+	}] {
+	    return
+	}
+    } elseif ![tixStrEq $value ""] {	
+	if [catch {
+	    # This needs to be catch'ed because the default itemtype may
+	    # not support the -text option
+	    #
+	    $w set $x $y -text $value
+	}] {
+	    return
+	}
+    } else {
+	return
+    }
+
+    set editDoneCmd [$w cget -editdonecmd]
+    if ![tixStrEq $editDoneCmd ""] {
+	uplevel #0 $editDoneCmd $x $y
+    }
 }
 
 
-sub DoneEdit
-  {
-    my ($w, $x, $y, @args) = @_;
-
-    my $edit = $w->privateData()->{__EDIT__};
-    $edit->configure(-command=>undef);
-    $edit->unpost;
-
-    # FIX xxx
-    # set value [tixEvent value]
-    my $value = $edit->get;
-    if ($w->info('exists', $x, $y))
-      {
-	Tk::catch { $w->entryconfigure($x, $y, -text=>$value) };
-        if ($@)
-	  {
-	    return
-	  }
-      }
-    elsif ( length($value) )
-      {
-	# This needs to be catch'ed because the default itemtype may
-	# not support the -text option
-	#
-	Tk::catch { $w->set($x,$y,-text $value); };
-        if ($@)
-          {
-	    return;
-          }
-      }
-    else
-      {
-	return;
-      }
-    $w->Callback('-editdonecmd' => $x, $y);
-  }
-
 1;
+
 __END__
