@@ -1,11 +1,11 @@
-# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-1998 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::After;
 use Carp;
 
 use vars qw($VERSION);
-$VERSION = '3.013'; # $Id: //depot/Tk8/Tk/After.pm#13 $
+$VERSION = '3.008'; # $Id: //depot/Tk8/Tk/After.pm#8$
 
 sub _cancelAll
 {
@@ -58,7 +58,7 @@ sub cancel
  my $w   = $obj->[0];
  if ($id)
   {
-   $w->Tk::after('cancel'=> $id);
+   $w->Tk::after('cancel'=> $id); 
    delete $w->{_After_}{$id};
    $obj->[1] = undef;
   }
@@ -86,3 +86,33 @@ sub once
 1;
 __END__
 
+=head1 NAME
+
+Tk::After - support class for Tk::Widget::after
+
+=for category Binding Events and Callbacks
+
+=head1 SYNOPSIS
+
+  $id = $widget->after(time,callback);
+  $id = $widget->afterIdle(callback);
+  $widget->afterCancel($id);
+
+  $id = $widget->repeat(period,callback);
+
+Internally this class is used to implement above.
+
+  $id = Tk::After->new($widget,$time,'method',callback);
+  $id->cancel;
+
+=head1 DESCRIPTION
+
+This class is a wrapper used by Tk::Widget::after to auto-cancel
+after calls when a widget is destroyed.
+
+I<callback> is a normal perl/Tk callback. Method is either I<'once'> 
+or I<'repeat'>
+
+This is first attempt at the code and interface is likely to change.
+So for the time being at least use $widget->after(...) interface.
+
