@@ -29,20 +29,13 @@ sub items {
     $iinfo{areaY2} = 0;
     $iinfo{restore_cmd} = '';
     
-                         
-    if (0) {
     # Display a 3x3 rectangular grid.
-    $c->create(qw/rect  0c  0c 30c 24c -width 2/);
-    $c->create(qw/line  0c  8c 30c  8c -width 2/);
-    $c->create(qw/line  0c 16c 30c 16c -width 2/);
-    $c->create(qw/line 10c  0c 10c 24c -width 2/);
-    $c->create(qw/line 20c  0c 20c 24c -width 2/);
-    } else {
-     # Display a rectangular grid.
-     $c->create(qw/grid 0c  0c 10c 8c -width 2 -lines 1/);
-     $c->create(qw/grid 0c  0c 5c  4c -lines 1 -dash ./);
-     $c->create(qw/grid 0c  0c 10m 8m -width 1/);
-    }
+
+    $c->create(qw/rect 0c 0c 30c 24c -width 2/);
+    $c->create(qw/line 0c 8c 30c 8c -width 2/);
+    $c->create(qw/line 0c 16c 30c 16c -width 2/);
+    $c->create(qw/line 10c 0c 10c 24c -width 2/);
+    $c->create(qw/line 20c 0c 20c 24c -width 2/);
 
     my $font1 = '-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*';
     my $font2 = '-*-Helvetica-Bold-R-Normal--*-240-*-*-*-*-*-*';
@@ -64,8 +57,8 @@ sub items {
     $c->create(qw/text 5c .2c -text Lines -anchor n/);
     $c->create(qw/line 1c 1c 3c 1c 1c 4c 3c 4c -width 2m/, -fill => $blue,
 	       qw/-cap butt -join miter -tags item/);
-    $c->create(qw/line 4.67c 1c 4.67c 4c -arrow last -activedash - -tags item/);
-    $c->create(qw/line 6.33c 1c 6.33c 4c -arrow both -activedash . -tags item/);
+    $c->create(qw/line 4.67c 1c 4.67c 4c -arrow last -tags item/);
+    $c->create(qw/line 6.33c 1c 6.33c 4c -arrow both -tags item/);
     $c->create(qw/line 5c 6c 9c 6c 9c 1c 8c 1c 8c 4.8c 8.8c 4.8c 8.8c 1.2c
                8.2c 1.2c 8.2c 4.6c 8.6c 4.6c 8.6c 1.4c 8.4c 1.4c
 	       8.4c 4.4c -width 3 -tags item -fill/ => $red);
@@ -128,10 +121,10 @@ sub items {
 	       -text => 'Stippled characters', qw/-tags item/);
 
     $c->create(qw/text 5c 16.2c -text Arcs -anchor n/);
-    $c->create(qw/arc 0.5c 17c 7c 20c/, -fill => $green, qw/-outline black/,
-	       -stipple => '@'.Tk->findINC('demos/images/grey.25'),
-	       qw/-start 45 -extent 270 -style pieslice -tags item/);
+    $c->create(qw/arc 0.5c 17c 7c 20c/, -fill => $green, qw/-outline black
+	       -start 45 -extent 270 -style pieslice -tags item/);
     $c->create(qw/arc 6.5c 17c 9.5c 20c -width 4m -style arc/, -fill => $blue,
+	       -stipple => '@'.Tk->findINC('demos/images/grey.25'),
 	       qw/-start -135 -extent 270 -tags item/);
     $c->create(qw/arc 0.5c 20c 9.5c 24c -width 4m -style pieslice/,
 	       -fill => undef, -outline => $red,
@@ -179,8 +172,6 @@ sub items {
     # read the specially localized variable $Tk::event.  We'll use XEvent
     # first, and the variable from then on.
 
-    $c->CanvasBind('<<Copy>>',sub { print "Do Copy\n" });
-
     $c->CanvasBind('<1>' => sub {
 	my($c) = @_;
         my $e = $c->XEvent;
@@ -261,12 +252,7 @@ sub items_enter {
 	return;
     }
     my $fill = ($c->itemconfigure(qw/current -fill/))[4];
-    my $stipple = ($c->itemconfigure(qw/current -stipple/))[4];
-    if (defined $stipple) {
-	$iinfo->{restore_cmd} = "\$c->itemconfigure('current',
-            -stipple => '$stipple')";
-	$c->itemconfigure(qw/current -stipple /,'');
-    } elsif (($type eq 'rectangle' or $type eq 'oval' or $type eq 'arc')
+    if (($type eq 'rectangle' or $type eq 'oval' or $type eq 'arc')
 	    and not defined $fill) {
 	my $outline = ($c->itemconfigure(qw/current -outline/))[4];
 	$iinfo->{restore_cmd} = "\$c->itemconfigure('current',
