@@ -10,7 +10,6 @@
  *
  */
 
-#include "tkInt.h"
 #include "tixPort.h"
 #include "tixInt.h"
 #include "tixImgXpm.h"
@@ -21,7 +20,7 @@
  */
 
 static int		ImgXpmCreate _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *name, int argc, Tcl_Obj **objv,
+			    char *name, int argc, char **argv,
 			    Tk_ImageType *typePtr, Tk_ImageMaster master,
 			    ClientData *clientDataPtr));
 static ClientData	ImgXpmGet _ANSI_ARGS_((Tk_Window tkwin,
@@ -40,7 +39,7 @@ static void		ImgXpmCmdDeletedProc _ANSI_ARGS_((
 static void		ImgXpmConfigureInstance _ANSI_ARGS_((
 			    PixmapInstance *instancePtr));
 static int		ImgXpmConfigureMaster _ANSI_ARGS_((
-			    PixmapMaster *masterPtr, int argc, Tcl_Obj **objv,
+			    PixmapMaster *masterPtr, int argc, char **argv,
 			    int flags));
 static int		ImgXpmGetData _ANSI_ARGS_((Tcl_Interp *interp,
 			    PixmapMaster *masterPtr));
@@ -81,7 +80,6 @@ Tk_ImageType tixPixmapImageType = {
     ImgXpmDisplay,		/* displayProc */
     ImgXpmFree,			/* freeProc */
     ImgXpmDelete,		/* deleteProc */
-    NULL,			/* postscriptProc */ 
     (Tk_ImageType *) NULL	/* nextPtr */
 };
 
@@ -110,12 +108,12 @@ static int xpmTableInited = 0;
  *----------------------------------------------------------------------
  */
 static int
-ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
+ImgXpmCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
     Tcl_Interp *interp;		/* Interpreter for application containing
 				 * image. */
     char *name;			/* Name to use for image. */
     int argc;			/* Number of arguments. */
-    Tcl_Obj **objv;		/* Argument strings for options (doesn't
+    char **argv;		/* Argument strings for options (doesn't
 				 * include image name or type). */
     Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
     Tk_ImageMaster master;	/* Token for image, to be used by us in
@@ -124,22 +122,6 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
 				 * it will be returned in later callbacks. */
 {
     PixmapMaster *masterPtr;
-    int i;      
-
-#ifndef _LANG
-    char *argvbuf[10];
-    char **argv = argvbuf;
-
-    /*
-     * Convert the objv arguments into string equivalent.
-     */
-    if (argc > 10) {
-	argv = (char **) ckalloc(argc * sizeof(char *));
-    }
-    for (i = 0; i < argc; i++) {
-	argv[i] = TixGetStringFromObj(objv[i], NULL);
-    }        
-#endif /* _LANG */
 
     masterPtr = (PixmapMaster *) ckalloc(sizeof(PixmapMaster));
     masterPtr->tkMaster = master;
@@ -154,7 +136,7 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
     masterPtr->isDataAlloced = 0;
     masterPtr->instancePtr = NULL;
 
-    if (ImgXpmConfigureMaster(masterPtr, argc, objv, 0) != TCL_OK) {
+    if (ImgXpmConfigureMaster(masterPtr, argc, argv, 0) != TCL_OK) {
 	ImgXpmDelete((ClientData) masterPtr);
 	return TCL_ERROR;
     }
@@ -185,11 +167,11 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
  *----------------------------------------------------------------------
  */
 static int
-ImgXpmConfigureMaster(masterPtr, argc, objv, flags)
+ImgXpmConfigureMaster(masterPtr, argc, argv, flags)
     PixmapMaster *masterPtr;	/* Pointer to data structure describing
 				 * overall pixmap image to (reconfigure). */
     int argc;			/* Number of entries in argv. */
-    Tcl_Obj **objv;		/* Pairs of configuration options for image. */
+    char **argv;		/* Pairs of configuration options for image. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget,
 				 * such as TK_CONFIG_ARGV_ONLY. */
 {
@@ -202,7 +184,7 @@ ImgXpmConfigureMaster(masterPtr, argc, objv, flags)
     oldId   = masterPtr->id;
 
     if (Tk_ConfigureWidget(masterPtr->interp, Tk_MainWindow(masterPtr->interp),
-	    configSpecs, argc, objv, (char *) masterPtr, flags)
+	    configSpecs, argc, argv, (char *) masterPtr, flags)
 	    != TCL_OK) {
 	return TCL_ERROR;
     }
@@ -386,7 +368,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
      * to skip the leading blanks is good for using in-line XPM data in TCL
      * scripts
      */
-    while (isspace(UCHAR(*string))) {
+    while (isspace(*string)) {
 	++ string;
     }
 
@@ -462,7 +444,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
 		continue;
 	    }
 
-	    if (isspace(UCHAR(*p)) || *p == ',') {
+	    if (isspace(*p) || *p == ',') {
 	    }
 	    else if (*p == '}') {
 		break;
@@ -571,34 +553,34 @@ GetType(colorDefn, type_ret)
     char * p = colorDefn;
 
     /* skip white spaces */
-    while (*p && isspace(UCHAR(*p))) {
+    while (*p && isspace(*p)) {
 	p ++;
     }
 
     /* parse the type */
     if (p[0] != '\0' && p[0] == 'm' &&
-	p[1] != '\0' && isspace(UCHAR(p[1]))) {
+	p[1] != '\0' && isspace(p[1])) {
 	*type_ret = XPM_MONO;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 'g' &&
 	     p[1] != '\0' && p[1] == '4' &&
-	     p[2] != '\0' && isspace(UCHAR(p[2]))) {
+	     p[2] != '\0' && isspace(p[2])) {
 	*type_ret = XPM_GRAY_4;
 	p += 3;
     }
     else if (p[0] != '\0' && p[0] == 'g' &&
-	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
+	     p[1] != '\0' && isspace(p[1])) {
 	*type_ret = XPM_GRAY;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 'c' &&
-	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
+	     p[1] != '\0' && isspace(p[1])) {
 	*type_ret = XPM_COLOR;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 's' &&
-	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
+	     p[1] != '\0' && isspace(p[1])) {
 	*type_ret = XPM_SYMBOLIC;
 	p += 2;
     }
@@ -635,7 +617,7 @@ GetColor(colorDefn, colorName, type_ret)
     }
 
     /* skip white spaces */
-    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
+    while (*colorDefn && isspace(*colorDefn)) {
 	colorDefn ++;
     }
 
@@ -644,7 +626,7 @@ GetColor(colorDefn, colorName, type_ret)
     while (1) {
 	int dummy;
 
-	while (*colorDefn && !isspace(UCHAR(*colorDefn))) {
+	while (*colorDefn && !isspace(*colorDefn)) {
 	    *p++ = *colorDefn++;
 	}
 
@@ -656,7 +638,7 @@ GetColor(colorDefn, colorName, type_ret)
 	    /* the next string should also be considered as a part of a color
 	     * name */
 	    
-	    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
+	    while (*colorDefn && isspace(*colorDefn)) {
 		*p++ = *colorDefn++;
 	    }
 	} else {
